#!/usr/bin/perl

###############################################################################
#
# This file can be distributed and modified freely, as long as this original
# header is not modified.
#
# Author:         Niversoft ides logicielles
# Contact:        info at niversoft dot com
# Date:           October 2004
# Last update:    June 2015
#
# This code is provided AS IS, without any warranty of any kind.
#
#  This script scans the last Communigate Pro log file and searches it for
#  "unknown user" messages during SMTP transaction. When it finds over
#  $tolerate message from the same IP, this IP is added to the CGP Blacklisted
#  IP Addresses. This is to prevent dictionary attacks over multiple SMTP
#  sessions.
#
#  Set this script as a cron job. The cron job should run at least once per log
#  rotation, but you can run it more often. I suggest you reduce the maximum
#  log size or period to reduce the run time of this script.
#
###############################################################################
#


print "Dictionary Attack Filter v1.5 by Niversoft\n";

use CLI;
use strict;
use Socket;

$CGP::SECURE_LOGIN = 0;

#  IPs you don't want to be added in the Blacklisted IP Addresses.
#  Client IPs, Lan IPs, and White Hole IP will be automatically loaded from
#  CGPro and never be added to the Blacklisted list
my @protectedips = (
        "192.168.0.1",
        "127.0.0.1"
);

my $tolerate    = 2;                                # number of tolerated attempts before blacklisting
my $checklast   = 1;                                # gather the LastAddress of each account and prevent those addresses from being blacklisted
my $logdir      = "/var/Communigate/SystemLogs";    # your log path
my $server      = "localhost";                      # your server
my $login       = "postmaster";                     # your login
my $password    = "password";                       # your password
my $altlog      = 1;                                # set to 1 if you want the filter to create a log of all IPs added to CGPro
my $altlogfile  = "/var/Communigate/SystemLogs/blacklisted.ips"; # log file

my $file = "";
if (scalar @ARGV > 0)
{
   if (-f $ARGV[0])
   {
      $file = $ARGV[0];
   }
   else
   {
      print "File not found $ARGV[0]\n";
      exit;
   }
}
else
{

   opendir LOGDIR, $logdir;

   # On Windows, you must start Communigate Pro with the --SharedFiles flag. Without this flag,
   # the latest log file is locked.

   my @logs = ();
   foreach my $file (readdir(LOGDIR))
   {
      push @logs, $file if ($file =~ /^\d+-\d+-\d+.*\.log$/);
   }
   closedir LOGDIR;

   my $lastlog = (sort {$b cmp $a} @logs)[0];
   $file = "$logdir/$lastlog";
}

print "Opening $file\n";

open(LOGFILE, "$file");
my %rejectedcount;
my %rejectedlist;
foreach my $line (<LOGFILE>)
{
    if ( ($line =~ /\d (POP|SMTP|IMAP)-\d+\((.*)\).*rejected: unknown user account/i)
      or ($line =~ /\d (POP|SMTP|IMAP)-\d+\((.*)\).*rejected: illegal name for an account/i)
      or ($line =~ /\d (POP|SMTP|IMAP)-\d+\((.*)\).*error code=unknown user account/i)
      or ($line =~ /\d (POP|SMTP|IMAP)-\d+\((.*)\).*error code=account is not available on this system/i)
      or ($line =~ /\d (POP|SMTP|IMAP).?-\d+\((.*?)\).*unknown user account/i)
      or ($line =~ /\d (POP|SMTP|IMAP).?-\d+\((.*?)\).*illegal name for an account/i)
    )
    {
      my $addrname = $2;
      $addrname =~ s/\[|\]//g;
      my $address = gethostbyname($addrname);
      if (length($address) == 4)
      {
         $rejectedcount{inet_ntoa($address)}++;
         $rejectedlist{inet_ntoa($address)} = $addrname;
      }
      else
      {
         print "Unable to resolve IP for '$addrname', entry discarded\n";
      }
    }
}
close(LOGFILE);

my $cli = new CGP::CLI
( {
    PeerAddr=>$server,
    PeerPort=>106,
    login=>$login,
    password=>$password
})  ||  die "Can't login to CGPro: ".$CGP::ERR_STRING."\n";


my %protectedips;
foreach (@protectedips)
{
   $protectedips{$_} = "";
}

SplitIPList($cli->GetClientIPs(), \%protectedips);
SplitIPList($cli->GetWhiteHoleIPs(), \%protectedips);
SplitIPList($cli->GetLANIPs(), \%protectedips);

foreach (keys %protectedips)
{
    delete $rejectedlist{$_} if (defined $rejectedcount{$_});
}

if ($checklast && scalar keys %rejectedlist)
{
   %protectedips = ();

   my $domains = $cli->ListDomains();
   foreach my $domain (sort @$domains)
   {
      my $accounts = $cli->ListAccounts($domain);

      foreach my $account (sort keys %{$accounts})
      {
         my $ip = $cli->GetAccountInfo("$account\@$domain", "LastAddress");
         $ip =~ s/[^\d\.]//g;
         delete $rejectedlist{$ip} if (defined $rejectedcount{$ip});
      }
   }
}

if (!scalar keys %rejectedlist)
{
   print "No dictionary attack found\n";
   exit;
}

my %BlacklistedIPs = ();
SplitIPList($cli->GetBlacklistedIPs(), \%BlacklistedIPs);


if ($altlog)
{
   open OUTLOG, ">>$altlogfile" or print STDERR "Can't open external log file $altlogfile\n";
}
my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
$year+=1900;
$mon+=1;
$hour = "0$hour" if ($hour < 10);
$min = "0$min" if ($min < 10);
$sec = "0$sec" if ($sec < 10);
$mon = "0$mon" if ($mon < 10);
$mday = "0$mday" if ($mday < 10);
my $somefound = 0;
foreach (keys %rejectedlist)
{
   if (!defined $BlacklistedIPs{$_} and $rejectedcount{$_} > $tolerate)
   {
      my $pad = " " x (15 - length($_));
      $BlacklistedIPs{$_} = $rejectedlist{$_};
      print OUTLOG "$year-$mon-$mday $hour:$min:$sec $_$pad ; $rejectedlist{$_};\n" if ($altlog);
      $somefound++;
   }
}
close OUTLOG;

if (!$somefound)
{
   print "No additional ip to blacklist\n";
   exit;
}

print "Setting Blackilisted IPs\n";
my @BlacklistedIPs;
foreach (keys %BlacklistedIPs)
{
   my $pad = " " x (15 - length($_));
   push @BlacklistedIPs, "$_$pad ; $BlacklistedIPs{$_}";
}

$cli->SetBlacklistedIPs(join( "\\e", @BlacklistedIPs));

sub SplitIPList
{
   my $iplist = shift;
   my $hash = shift;
   foreach my $splitted (split /\\e/, $iplist)
   {
      my @sp = split / *; */, $splitted;
      $sp[1] = "" if(!defined $sp[1]);
      $hash->{$sp[0]} = $sp[1];
   }
}